import os
import wave
import tempfile
import numpy as np
from scipy.io.wavfile import read as read_wave
from scipy.io.wavfile import write as write_wave
import subprocess
from threading import Thread

try:
    from .soundsample import SoundSample
    from .soundexplorer import SoundExplorer
except ImportError:
    from backend.soundsample import SoundSample
    from backend.soundexplorer import SoundExplorer


def play(filename):
    try:
        subprocess.call(['open', filename])
    except FileNotFoundError:
        t = Thread(target=os.system,
                   args=['"' + filename + '"'])
        t.start()

class SimpleSound(object):
    def __init__(self, arg=None):
        # arg is either a filename (a string) or a length (an int).
        if type(arg) == str:
            self.filename = arg
            self.decode_wav(arg)
        elif type(arg) == int:
            self.samples = [SoundSample(0) for i in range(arg)]
            self.sample_rate = 22050
        else:
            raise Exception('Could not create Sound. Please specify either a length or a filename.')
    
    def getLength(self):
        return len(self.samples)

    def getSamples(self):
        return self.samples

    def getSample(self, index):
        return self.samples[index]
    
    def decode_wav(self, filename):
        self.sample_rate, samples = read_wave(filename)
        self.samples = [SoundSample(d) for d in samples]
        self.length = len(self.samples)
    
    def write(self, filename, begin=None, end=None):
        if not begin:
            begin = 0
        if not end:
            end = len(self.samples)
        
        samples = np.array([s.getValue() for s in self.samples[begin : end]]).astype('int16')
        write_wave(filename, self.sample_rate, samples)

    def play(self, begin=None, end=None):
        '''Play the sound.'''
        _, tmp_file = tempfile.mkstemp()
        self.write(tmp_file + '.wav', begin, end)
        play_file = (tmp_file + '.wav').replace('\\', '/')
        play(play_file)

    def explore(self):
        explorer = SoundExplorer(self)
        explorer.show()
