import random
from backend.simplepicture import SimplePicture
from backend.color import Color
from cs116 import *

class Picture(SimplePicture):
    def setToColor(self, color):
        for pixel in self.getPixels():
            pixel.setColor(color)
    
    def posterize(self):
        for p in self.getPixels():
            r = p.getRed()
            g = p.getGreen()
            b = p.getBlue()
            if r < 64:
                p.setRed(31)
            elif r > 63 and r < 128:
                p.setRed(95)
            elif r > 127 and r < 192:
                p.setRed(159)
            else:
                p.setRed(223)
                
            if g < 64:
                p.setGreen(31)
            elif g > 63 and g < 128:
                p.setGreen(95)
            elif g > 127 and g < 192:
                p.setGreen(159)
            else:
                p.setGreen(223)
                
            if b < 64:
                p.setBlue(31)
            elif b > 63 and b < 128:
                p.setBlue(95)
            elif b > 127 and b < 192:
                p.setBlue(159)
            else:
                p.setBlue(223)
    
    #count the number of pixels of a given color in a picture
    def countPixelsOfColor(self, color):
        count = 0
        for pixel in self.getPixels():
            if pixel.getColor() == color:
                count = count + 1
        return count
    
    def copy(self):
        result = Picture(self.getWidth(), self.getHeight())
        pixels1 = self.getPixels()
        pixels2 = result.getPixels()
        for i in range(0, len(pixels1)):
            color = pixels1[i].getColor()
            pixels2[i].setColor(color)
        return result
    
    def toSound(self):
        pixels = self.getPixels()
        sound = Sound(len(pixels))
        samples = sound.getSamples()
        for i in range(0, len(pixels)):
            if pixels[i].getRed() > 200:
                samples[i].setValue(1000)
            elif pixels[i].getBlue() > 200:
                samples[i].setValue(-1000)
            else:
                samples[i].setValue(0)
        return sound
    
    def repeatFirstHalf(self):
        pixels = self.getPixels()
        offset = len(pixels) // 2
        for i in range(0,len(pixels)//2):
            pixels[offset+i].setColor(pixels[i].getColor())
    
    def draw(self, xoffset, y, width):
        for xindex in range(0,width):
            pixel = self.getPixel(xoffset+xindex,y)
            pixel.setColor(black)
            
    def crop(self, xoffset, yoffset, width, height):
        result = Picture(width, height)
        for xindex in range(0, width):
            for yindex in range(0, height):
                pixel1 = self.getPixel(xoffset+xindex, yoffset+yindex)
                # complete the code to get the correct pixel pixel2 from result
                pixel2.setColor(pixel1.getColor())
        return result
                
    def flip(self):
        result = Picture(self.getWidth(), self.getHeight())
        for xindex in range(0, self.getWidth()):
            for yindex in range(0, self.getHeight()):
                pixel = self.getPixel(xindex, yindex)
                resultPixel = result.getPixel(self.getWidth()-1-xindex, yindex)
                resultPixel.setColor(pixel.getColor())
        return result
    
    def edges(self, threshold):
        result = Picture(self.getWidth(), self.getHeight())
        for x in range(0, self.getWidth()-1):
            for y in range(0, self.getHeight()-1):
                pixel = self.getPixel(x, y)
                pixelRight = self.getPixel(x+1, y)
                pixelDown = self.getPixel(x, y+1)
                pixelColor = pixel.getColor()
                rightColor = pixelRight.getColor()
                downColor = pixelDown.getColor()
                if (pixelColor.colorDistance(rightColor) > threshold or
                    pixelColor.colorDistance(downColor) > threshold):
                    result.getPixel(x, y).setColor(black)
        return result
    
    def blob(self, x, y):
        # if the pixel is outside the picture then don't count it
        if x < 0 or y < 0 or x > self.getWidth() or y > self.getHeight():
            return 0
        # if the pixel is white(ish) then we don't count it
        elif self.getPixel(x, y).getColor().colorDistance(white) < 10:
            return 0
        else:
            # mark the pixel as white so it doesn't get counted again
            self.getPixel(x, y).setColor(green)
            # count the pixel plus the blob counts of all its neighbors
            return (1 + self.blob(x-1, y) + self.blob(x+1, y) +
                    self.blob(x, y-1) + self.blob(x, y+1))
    
